%*** Calculates S11 and S21 parameters generated by CUDA TLM-SCN code
%***  Reads in Reference data, Probe 1 data , and Probe 2 data and
%***    calculates the FFT of each. Then Calculates the S21 and S11
%***    parameters with the FFT results. S11 and S21 are then inserted into
%***    an excel file, "MatlabFFT Results.xlsx", that is already formated with the
%***    appropriate greaphs. This excel file also has Mefisto results that are
%***    used to compare the CUDA results of a WR-28 Bandpass filter. 

%*** IMPORTANT: the "MatlabFFT Results.xlsx" file must be closed before
%***  executing this script

%strAddressLoc = 'C:\Documents and Settings\All Users\Application Data\NVIDIA Corporation\NVIDIA CUDA SDK\projects\Fil TLM 64 v14 OOP design\Data';
%strAddressLoc = 'C:\Documents and Settings\All Users\Application Data\NVIDIA Corporation\NVIDIA CUDA SDK\projects\Fil TLM 64 v15 OOP design with new kernels\Data';
disp(' Reading Probe Sample data')
strAddressLoc =''; % a little redundant now that matlab code placed in same directory 
strRef_csv = strcat(strAddressLoc,'\Sampler_0_REF.csv'); % concatenate directory path to file name
strProbe1_csv = strcat(strAddressLoc,'\Sampler_1_Probe1.csv');
strProbe2_csv = strcat(strAddressLoc,'\Sampler_2_Probe2.csv');


%dataNum = 16000;
dataNum = csvread(strRef_csv, 0, 4,[0,4,0,4]); % number of samples
Ts = csvread(strRef_csv, 2, 1,[2,1,2,1]); % Sampling Time (sec)

%L = csvread(strRef_csv, 0, 3,[0,3,0,3]);
% usage: M = csvread(filename, row, col, range)
Cuda_Ref = csvread(strRef_csv, 1, 2, [1,2,dataNum,2]); % read Ref data from CSV file
Cuda_Probe1 = csvread(strProbe1_csv, 1, 2, [1,2,dataNum,2]);% read Probe1 data from CSV file
Cuda_Probe2 = csvread(strProbe2_csv, 1, 2, [1,2,dataNum,2]);% read Probe2 data from CSV file

%[L,junk] = size(Cuda_Ref);
disp(' S-Param Setup')
L = length(Cuda_Ref); 
NFFT = 2^nextpow2(L); % Next power of 2 from length of y
NFFT = 4*NFFT; %  Expanding number of time domain data points to create a finer resolution of FFT data points
%NFFT = 64000;
%Ts = Cuda_Time(2);
Fs = 1/Ts; % Sampling Frequency (Hz)
fft_Ref = fft(Cuda_Ref,NFFT);  % FFT calculation
fft_Probe1 = fft(Cuda_Probe1,NFFT);% FFT calculation
fft_Probe2 = fft(Cuda_Probe2,NFFT);% FFT calculation

f = Fs/2*linspace(0,1,NFFT/2+1); % create an array of frequency spectra
%f = Fs/2*linspace(0,1,NFFT/2);

%*** Calculate S11 and S21 parameters ***
disp(' S-Param Calculation')
for x=1:L
    S11(x) = ((fft_Probe1(x) - fft_Ref(x))/fft_Ref(x));
    S21(x) = (fft_Probe2(x)/fft_Ref(x));
end

strSParamResults = strcat(strAddressLoc,'\MatlabFFT Results.xlsx');% Concatonate name of results excel file
xlswrite(strSParamResults, {'CUDA S-Params'}, 'B2:B2'); 
strTitles = {'Freq', 'Cuda-S11', 'Cuda-S21'};
disp(' Excel Write: Titles')
xlswrite(strSParamResults, strTitles, 'A4:C4');
disp(' Excel Write: Freq Array')
xlswrite(strSParamResults, f', 'A5:A5000'); % Write freq array to "MatlabFFT Results.xlsx"
disp(' Excel Write: S11 array')
xlswrite(strSParamResults, abs(S11)', 'B5:B5000');% Write S11 array to "MatlabFFT Results.xlsx"
disp(' Excel Write: S21 array')
xlswrite(strSParamResults, abs(S21)', 'C5:C5000');% Write S21 array to "MatlabFFT Results.xlsx"
disp(' Done')
